; Second Life Installer Script
; Copyright (C) 2007 Dale Glass
;
; This program is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program.  If not, see <http://www.gnu.org/licenses/>.
!include "version.nsh"
!include "settings.nsh"

!define LINDEN_REG_KEY "SOFTWARE\Linden Research, Inc."
!define DOWNLOAD_SITE "http://daleglass.net/installer/"

; Process the whole manifest, running a function per line
; Function must take the arguments from the stack, in this
; order:
;
; File
; MD5 sum
; Params
!macro process_manifest COMMAND

	FileOpen $0 "$INSTDIR\Manifest" r
	FileOpen $0 "$INSTDIR\Manifest" r
	
	FileRead $0 $R0 ; File count
	IfErrors ManifestReadError
	StrCpy $R0 $R0 -2

	FileRead $0 $R1 ; uncompressed total
	IfErrors ManifestReadError
	StrCpy $R1 $R1 -2

	FileRead $0 $R2 ; compressed total
	IfErrors ManifestReadError
	StrCpy $R2 $R2 -2

	StrCpy $7 0 ; Copied
	
	IntOp $R2 $R2 / 100
	
	ReadLine:
	        ; REQUIRED! ClearErrors needed here, or the loop may prematurely terminate
	        ; due to things happening inside the command being called.
	        
	        ClearErrors
        	FileRead $0 $1 ; File
		IfErrors EndRead
		StrCpy $1 $1 -2

        	FileRead $0 $2 ; MD5 sum of uncompressed data
		IfErrors ManifestReadError
		StrCpy $2 $2 -2

        	FileRead $0 $3 ; size of uncompressed data
		IfErrors ManifestReadError
		StrCpy $3 $3 -2

        	FileRead $0 $4 ; SHA-1 of compressed file
		IfErrors ManifestReadError
		StrCpy $4 $4 -2

        	FileRead $0 $5 ; size of compressed file
		IfErrors ManifestReadError
		StrCpy $5 $5 -2

        	FileRead $0 $6 ; Params
		IfErrors ManifestReadError
		StrCpy $6 $6 -2

		push $1
		push $2
		push $4
		push $6

		Call ${COMMAND}

		; Calculate progress
		IntOp $7 $7 + $5 ; Processed so far (compressed)
		IntOp $8 $7 / $R2 ; Calculate %
		
!ifdef USE_REALPROGRESS
		RealProgress::SetProgress /NOUNLOAD $8
!endif

		goto ReadLine
	EndRead:
	DetailPrint "Finished installing files"
	
	FileClose $0
        Goto FinishInstall

        ManifestReadError:
                MessageBox MB_OK|MB_ICONSTOP "Failed to read manifest. $\n$\n \
					      The installer may be corrupt."
                Quit

        FinishInstall:
        
!macroend

; Installs a file
; Arguments:
;       File
;       MD5 sum
;       Params
;
; The function will check whether the file is already there. If it is, and the MD5 sum
; is correct, then nothing is done.
;
; If the file can't be found, or the MD5 sum is wrong, it will scan all the known SL
; installations and try to get the file from there. MD5 sums will checked to ensure
; an exact match.
;
; If this also fails, it will download it from the server, if the file is redistributable.
;
; If the file can't be downloaded because the download itself fails, or because it's not
; redistributable, the installation will fail.
Function install_file

	; stack = file,md5,sha1,params
	
	Exch $R0 ; Params; stack = file,md5,sha1,old$R0
	Exch             ; stack = file,md5,old$R0,sha1
	Exch $R3 ; sha1  ; stack = file,md5,old$R0,old$R3
	Exch 2           ; stack = file,old$R3,old$R0,md5
	Exch $R1 ; md5   ; stack = file,old$R3,old$R0,old$R1
	Exch 3           ; stack = old$R1,old$R3,old$R0,file
	Exch $R2 ; file  ; stack = old$R1,old$R3,old$R0,old$R2
	
	
	
	
;	Exch             ; stack = file,old$R0, md5
;	Exch $R1 ; MD5   ; stack = file,old$R0, old$R1
;	Exch 2           ; stack = old$R1, old$R0, file
;	Exch $R2 ; file  ; stack = old$R1, old$R0, $oldR2
;
;	Exch 2           ; stack = old$R2, old$R0, $oldR1
;	Exch             ; stack = old$R2, old$R1, $oldR0
	
	
	Push $0
	Push $1
	Push $2
	Push $3

	DetailPrint "Installing $R2"

			
	IfFileExists "$INSTDIR\$R2" 0 TryFind
	; File exists, check if it's the right one
	LogText "Checking MD5 for $INSTDIR\$R2"
        md5dll::GetMD5File "$INSTDIR\$R2"
        Pop $3
        StrCmp $R1 $3 done ; If MD5 matches, there's nothing else to do
        
	TryFind:
	        ; File not found, or not the right one.
	        DetailPrint "File not found in install destination"
	        
	        StrCpy $0 0
	        
;		DetailPrint "Looking for installs at ${LINDEN_REG_KEY}"

	        CheckInstall:
	                ; Fetch location of SL install
	                
		        EnumRegKey $1 HKLM "${LINDEN_REG_KEY}" $0
		        StrCmp $1 "" CheckInstallEnd
			IntOp $0 $0 + 1
		        
		        ;DetailPrint "Checking install at $1"
		        
		        ; Read location from registry
		        ReadRegStr $2 HKLM "${LINDEN_REG_KEY}\$1" ""
		        IfFileExists "$2\$R2" 0 CheckInstall
		        
		        ; File exists, check md5sum
		        LogText "Checking MD5 for $2\$R2"
	                md5dll::GetMD5File "$2\$R2"
        		Pop $3
		        StrCmp $R1 $3 0 CheckInstall
		        
		        ; Copy file, done
			LogText "Copying $2\$R2 to $INSTDIR\$R2"
			CopyFiles /silent "$2\$R2" "$INSTDIR\$R2"
			Goto done
        
	        	Goto CheckInstall
	        
                CheckInstallEnd:

	; Check if the file is not redistributable
	StrCmp $R0 "noredist" NoRedist
	
	DetailPrint "$R2 not found locally, downloading"
	
	CreateDirectory "$TEMP\SecondLife"
	
	;IfFileExists "$TEMP\SecondLife\$R1.tar.bz2" download_ok 0
	
	;NSISdl::download "${DOWNLOAD_SITE}/$R2/$R1.tar.bz2" "$TEMP\SecondLife\$R1.tar.bz2"
	LogText "Downloading ${DOWNLOAD_SITE}/$R2/$R1.${COMPRESSED_EXT} to $TEMP\SecondLife\$R1.${COMPRESSED_EXT}"
	metadl::download /SHA1 $R3 "${DOWNLOAD_SITE}/$R2/$R1.${COMPRESSED_EXT}" "$TEMP\SecondLife\$R1.${COMPRESSED_EXT}"
	
	Pop $R0 ;Get the return value
	StrCmp $R0 "success" download_ok
	
	LogText "Removing $TEMP\SecondLife\$R1.${COMPRESSED_EXT}"
	Delete "$TEMP\SecondLife\$R1.${COMPRESSED_EXT}"
	MessageBox MB_OK|MB_ICONSTOP "Download failed: $R0"
	Quit

	download_ok:
	        ; Download done, uncompress
		DetailPrint "Uncompressing $R2"

		!ifdef USE_UNTGZ
		untgz::extract /NOUNLOAD -z${COMPRESSION_TYPE} "$TEMP\SecondLife\$R1.${COMPRESSED_EXT}"
		!else
                nsExec::ExecToLog 'lzma.exe d "$TEMP\SecondLife\$R1.${COMPRESSED_EXT}" "$R2"'
                Pop $0
                StrCmp $0 "error" UncompressError
		!endif
		
;		IfErrors UncompressError
		LogText "Uncompression OK"
		ClearErrors
		
	        ;Delete "$TEMP\SecondLife\$R1.tar.bz2"
	        
	        Goto done
	UncompressError:
		MessageBox MB_OK|MB_ICONSTOP "Uncompression of file $R2 failed"
		Quit
	NoRedist:
		MessageBox MB_OK|MB_ICONSTOP "The file $R2 was not found on your computer, and can't be downloaded because it's not redistributable.$\n$\n \
					      The viewer should work anyway, as it will use a distributable component instead. However, this has been \
					      less tested, and might be slower. If you experience problems, install the latest official SL viewer, \
					      and run this installer again."
					      

;		Quit
	done:

	
	
	Pop $3
	Pop $2
	Pop $1
	Pop $0
	
	;old$R1,old$R3,old$R0,old$R2
	Pop $R2
	Pop $R0
	Pop $R3
	Pop $R1

	;Pop $R0
	;Pop $R1
	;Pop $R2
FunctionEnd

; A bit wasteful, written to be compatible
; with install_file, and usable in the process_manifest macro.
Function un.remove_file
	Exch $R0 ; Params; stack = file,md5,old$R0
	Exch             ; stack = file,old$R0, md5
	Exch $R1 ; MD5   ; stack = file,old$R0, old$R1
	Exch 2           ; stack = old$R1, old$R0, file
	Exch $R2 ; file  ; stack = old$R1, old$R0, $oldR2

	Exch 2           ; stack = old$R2, old$R0, $oldR1
	Exch             ; stack = old$R2, old$R1, $oldR0
	
	Delete $R0
	
	Pop $R2
	Pop $R1
	Pop $R0
FunctionEnd

; Check that Second Life is installed
; This checks all the installs listed in the registry,
; and makes sure the .exe file is still there
Function detect_sl
	Push $0
	Push $1
	Push $2
	Push $3
	
	StrCpy $0 0

	DetailPrint "Looking for installs at ${LINDEN_REG_KEY}"
	CheckInstall:
	; Fetch location of SL install

	        EnumRegKey $1 HKLM "${LINDEN_REG_KEY}" $0
	        StrCmp $1 "" CheckInstallEnd
		IntOp $0 $0 + 1

	        DetailPrint "Checking install at $1"

	        ; Read location from registry
	        ReadRegStr $2 HKLM "${LINDEN_REG_KEY}\$1" ""
	        
	        ; Read name of .exe file
	        ReadRegStr $3 HKLM "${LINDEN_REG_KEY}\$1" "Exe"
	        
	        IfFileExists "$2\$3" Ok CheckInstall

	CheckInstallEnd:
		MessageBox MB_OK|MB_ICONSTOP "No Second Life installations were detected.$\n$\n \
		                             Due to redistribution restrictions on some components, this version of the viewer only comes with freely redistributable files and takes the rest from existing installs.$\n \
					     Please install an official Second Life client and try again."
		Abort

	Ok:

	Pop $3
	Pop $2
	Pop $1
	Pop $0
FunctionEnd
